/*--------------------------------------------------------------------------

   glvViewer class
   glvViewer.h v1.0 7/November/2003
   Copyright (C) 2003 J. Ribelles

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

  --------------------------------------------------------------------------*/

#ifndef GLVVIEWER_H
#define GLVVIEWER_H

#include <iostream.h>
#include <fstream.h>

#define glvParallel    1  /**< Parallel projection */
#define glvPerspective 2  /**< Perspective projection */

#define glvWhiteColor  1  /**< White color: 1, 1, 1 */
#define glvBlackColor  2  /**< Black color: 0, 0, 0 */
#define glvBlueColor   3  /**< Blue color:  0.24, 0.47, 0.60 */
#define glvMyOwnColor  4  /**< Color defined by the user */

//! This class implement an OpenGL Viewer.
/*! The goal is to provide a customizable environment 
    for displaying 3D OpenGL scenes and that it can be recreated.
    So, any user can display different scenes under the same rendering options.
    This is a very important condition in order to compare results
    obtained from different experiments realized by different users.
 */
class glvViewer {

 private:

  /* Initialization functions --------------------------------------------*/
  void glvMenu          (void);
  void glvEvents        (void);
  void glvRenderOptions (void);
  void glvRead          (ifstream &in);
  
  /* Callbacks of OpenGL functions ---------------------------------------*/
  static void _glvDisplay(void);
  static void _glvKeyboard (unsigned char key, int px, int py);
  static void _glvReshape (int width, int height);
  static void _glvMouse (int button, int state, int x, int y);
  static void _glvMotion (int newMouseX, int newMouseY);
  static void _glvIdle (void);
  static void _glvMenu (int option);

 public:

  /* Functions that an instance of this class can call -------------------*/
  glvViewer (void);                       // Constructor
  void glvInit (int argc, char **argv);  // Show the window
  void glvInitFLTK (void *w);            // Show the window for FLTK
  void glvLoop (void);                   // The main loop
  void glvSaveImage (void);              // Save a ppm image file
  void glvSaveOptions (void);            // Save the OpenGL options
  void glvReloadOptions (void);          // Reload the OpenGL options
  void glvSetFovy (float fovy);          // Set the fovy
  void glvSetProjection (int mode);
  int  glvGetProjection (void);
  void glvSetClearColor (int color);
  int  glvGetClearColor (void);

  /* Functions that an instance of this class should or might overload ---*/
  /** Your display callback. */
  virtual void glvMyDisplay(void) {};
  /** Your menu callback. id: is the id of the current menu. */
  virtual void glvMyMenu(int id) {};
  /** Your keyboard callback.
   * Variables are the same that those for the glutKeyboardFunc callback. */
  virtual void glvMyKeyboard(unsigned char key, int px, int py) {};
  /** Your render options function. 
   * You should write here other rendering options like fog,... */
  virtual void glvMyRenderOptions(void) {};
};

#endif
